@@ -132,6 +132,8 @@ class DecisionNode(Node):
         CPT_shape = []
         for par in self.parents:
             CPT_shape.append(len(self.parents[par].space))
+        if not CPT_shape:
+            CPT_shape.append(1)
         CPT_shape.append(len(self.space))
         self.CPT = np.zeros(CPT_shape)
         
@@ -169,24 +171,17 @@ class DecisionNode(Node):
             raise AttributeError('CPT for %s is just a zeros array' % self.name)
         ind = []
         indo = self.get_CPTindex(parentinput, valueinput=False)
-        idx = 0
         if not mode:
             cdf = np.cumsum(self.CPT[indo])
             cutoff = np.random.rand()
-#            if cdf[-1] < cutoff:
-#                raise ValueError(self.name, cdf[-1], cutoff, indo, self.CPT[indo]) 
-#            try:
-            idx = np.nonzero( cdf >= cutoff )[0][0]
-#            except IndexError:
-#                print self.name, cdf[-1], cutoff, indo, self.CPT[indo]    
+            idx = np.nonzero( cdf >= cutoff )[0][0]  
         else:
             idx = self.CPT[indo].argmax()
-        r = self.space[idx]
         if setvalue:
-            self.set_value(r)
+            self.set_valueindex(idx)
             return self.get_value()
         else:
-            return r
+            return self.space[idx]
         
     def randomCPT(self, mixed=False, setCPT=True):
         """Create a random CPT for the :class:`classes.DecisionNode` object
@@ -214,6 +209,7 @@ class DecisionNode(Node):
             else:
                 z.reshape((-1, shape_last))[0, y]=1
         else:
+            simplex_draws = np.random.dirichlet(np.ones(shape_last), np.prod(other_dims))
             M = 100000000
             x = randvars.randint.rvs(1, M, size=other_dims+(shape_last-1,))
             y = np.concatenate((np.zeros(other_dims+(1,)), x, \
@@ -241,7 +237,7 @@ class DecisionNode(Node):
         else:
             return z
         
-    def perturbCPT(self, noise, mixed=True, sliver=None, setCPT=True, \
+    def perturbCPT(self, noise, mixed=True, setCPT=True, \
                    returnweight=False):
         """Create a perturbation of the CPT attribute.
         
@@ -254,14 +250,6 @@ class DecisionNode(Node):
            weights shifted to other values. If mixed, then the perturbed CPT is 
            a mixed CPT with positive weight on all values. 
         :type mixed: bool
-        :arg sliver: Optional. Determines the values of the parents for which 
-           to perturb the current CPT. Keys are parent names. Values are parent 
-           values. If empty, the entire CPT is perturbed. If sliver is nonempty, 
-           but specifies values for only a subset of parents, the current values 
-           are used for the remaining parents. Note that for pure perturbations
-           the noise parameter is ignored and a different action is selected 
-           with probability 1.
-        :type sliver: dict
         
         .. note::
            
@@ -273,46 +261,26 @@ class DecisionNode(Node):
            True, then there is both CPT and weight output, and the weight is 
            first in the list.
         
-        """
+        """        
+#        :arg sliver: Optional. Determines the values of the parents for which 
+#           to perturb the current CPT. Keys are parent names. Values are parent 
+#           values. If empty, the entire CPT is perturbed. If sliver is nonempty, 
+#           but specifies values for only a subset of parents, the current values 
+#           are used for the remaining parents. Note that for pure perturbations
+#           the noise parameter is ignored and a different action is selected 
+#           with probability 1.
+#        :type sliver: dict
         copiedCPT = copy.copy(self.CPT)
         weight = 1
         if not mixed: #pure CPT
-            if not sliver: #perturbing the whole CPT
-                if returnweight:
-                    copiedCPT, weight = perturbpure(copiedCPT, noise, \
-                                                    returnweight)
-                else:
-                    copiedCPT = perturbpure(copiedCPT, noise, returnweight)
+            if returnweight:
+                copiedCPT, weight = perturbpure(copiedCPT, noise, \
+                                                returnweight)
             else:
-                for par in self.parents:
-                    if par not in sliver:
-                        sliver[par] = self.parents[par].get_value()
-                ind = self.get_CPTindex(sliver, valueinput=False)
-                zeroind = np.nonzero(copiedCPT[ind]==0)
-                if len(actind) == 0:
-                    raise ValueError('The specified sliver has no 0 entries')
-                newactind = np.random.uniform(0,len(actind))
-                copiedCPT[ind] = np.zeros(len(copiedCPT[ind]))
-                copiedCPT[ind][zeroind[newactind]] = 1
+                copiedCPT = perturbpure(copiedCPT, noise, returnweight)
         else: #mixed CPT
             randCPT = self.randomCPT(mixed=True, setCPT=False)
-            if not sliver: #perturbing the whole thing
-                copiedCPT = copiedCPT*(1-noise) + randCPT*noise
-            else: #perturbing only sliver
-                ind = []
-                for par in self.parents:
-                    if par in sliver:
-                        truth = [(x==sliver[par]).all() for x in \
-                                                        self.parents[par].space]
-                        ind.append(truth.index(True))
-                    else:
-                        value = self.parents[par].get_value()
-                        truth = [(x==value).all() for x in \
-                                                        self.parents[par].space]
-                        ind.append(truth.index(True))
-                indo = tuple(ind)
-                copiedCPT[indo] = copiedCPT[indo]*(1-noise) + \
-                                                            randCPT[indo]*noise
+            copiedCPT = copiedCPT*(1-noise) + randCPT*noise
         if setCPT:
             self.CPT[:] = copiedCPT
             if returnweight:
@@ -412,32 +380,6 @@ class DecisionNode(Node):
         else:
             return convert_2_pureCPT(copy.copy(self.CPT))
         
-#    def set_value(self, newvalue):
-#        """Set the current value of the DecisionNode object
-#        
-#        :arg newvalue: a legitimate value of the DecisionNode object, i.e. the 
-#           value must be in :py:attr:`classes.ChanceNode.space`.
-#        
-#        .. warning::
-#            
-#           When arbitrarily setting values, some children may have zero 
-#           probability given their parents. This means the logprob may be -inf. 
-#           If using, :py:meth:`seminfg.SemiNFG.loglike()`, this results in a 
-#           divide by zero error.
-#        
-#        """
-#        if type(newvalue==self.space[0]) is bool:
-#            if newvalue in self.space:
-#                self.set_value(newvalue)
-#            else:
-#                errorstring = "the new value is not in "+self.name+"'s space"
-#                raise ValueError(errorstring)
-#        elif any((newvalue==y).all() for y in self.space):
-#            self.set_value(newvalue)
-#        else:
-#            errorstring = "the new value is not in "+self.name+"'s space"
-#            raise ValueError(errorstring)  
-        
 def perturbpure(CPT, noise, returnweight):
     # Generate noise for each possible combination of parent node values and reshape
     oldCPT = copy.copy(CPT)
